@@ -6,14 +6,13 @@ import json
 import logging
 import re
 import warnings
+from collections import defaultdict
 
 import requests
 
 from spotipy.exceptions import SpotifyException
 from spotipy.util import Retry
 
-from collections import defaultdict
-
 logger = logging.getLogger(__name__)
 
 
@@ -263,8 +262,8 @@ class Spotify:
         if self.language is not None:
             headers["Accept-Language"] = self.language
 
-        logger.debug('Sending %s to %s with Params: %s Headers: %s and Body: %r ',
-                     method, url, args.get("params"), headers, args.get('data'))
+        logger.debug(f"Sending {method} to {url} with Params: "
+                     f"{args.get('params')} Headers: {headers} and Body: {args.get('data')!r}")
 
         try:
             response = self._session.request(
@@ -289,10 +288,8 @@ class Spotify:
                 msg = response.text or None
                 reason = None
 
-            logger.error(
-                'HTTP Error for %s to %s with Params: %s returned %s due to %s',
-                method, url, args.get("params"), response.status_code, msg
-            )
+            logger.error(f"HTTP Error for {method} to {url} with Params: "
+                         f"{args.get('params')} returned {response.status_code} due to {msg}")
 
             raise SpotifyException(
                 response.status_code,
@@ -317,7 +314,7 @@ class Spotify:
         except ValueError:
             results = None
 
-        logger.debug('RESULTS: %s', results)
+        logger.debug(f'RESULTS: {results}')
         return results
 
     def _get(self, url, args=None, payload=None, **kwargs):
@@ -1247,10 +1244,8 @@ class Spotify:
                 if they follow the playlist. Maximum: 5 ids.
 
         """
-        endpoint = "playlists/{}/followers/contains?ids={}"
-        return self._get(
-            endpoint.format(playlist_id, ",".join(user_ids))
-        )
+        endpoint = f"playlists/{playlist_id}/followers/contains?ids={','.join(user_ids)}"
+        return self._get(endpoint)
 
     def me(self):
         """ Get detailed profile information about the current user.
@@ -1497,7 +1492,7 @@ class Spotify:
         """ Get the current user's top artists
 
             Parameters:
-                - limit - the number of entities to return
+                - limit - the number of entities to return (max 50)
                 - offset - the index of the first entity to return
                 - time_range - Over what time frame are the affinities computed
                   Valid-values: short_term, medium_term, long_term
@@ -1766,6 +1761,11 @@ class Spotify:
         return self._get("recommendations", **params)
 
     def recommendation_genre_seeds(self):
+        warnings.warn(
+            "You're using `recommendation_genre_seeds(...)`, "
+            "which is marked as deprecated by Spotify.",
+            DeprecationWarning,
+        )
         """ Get a list of genres available for the recommendations function.
         """
         return self._get("recommendations/available-genre-seeds")
@@ -2066,11 +2066,9 @@ class Spotify:
     def _search_multiple_markets(self, q, limit, offset, type, markets, total):
         if total and limit > total:
             limit = total
-            warnings.warn(
-                "limit was auto-adjusted to equal {} as it must not be higher than total".format(
-                    total),
-                UserWarning,
-            )
+            warnings.warn(f"limit was auto-adjusted to equal {total} "
+                          f"as it must not be higher than total",
+                          UserWarning)
 
         results = defaultdict(dict)
         item_types = [item_type + "s" for item_type in type.split(",")]
